package org.KeyChallenge;

import javacard.framework.*;
import javacard.security.*;
import javacardx.apdu.ExtendedLength;

public class KeyChallengeApplet extends Applet implements ExtendedLength {

    private static final byte CHALLENGE_CLA =   (byte) 0xB0;
    private static final byte KEYINSERT_INS =   (byte) 0x01;
    private static final byte CHALLENGE_INS =   (byte) 0x02;


    private HMACKey serverKey = null;
    private HMAC hmacEngine = null;

    public static void install(byte[] bArray, short bOffset, byte bLength) {
        new KeyChallengeApplet();
    }

    public KeyChallengeApplet() {
        serverKey = new HMACKey(Signature.ALG_HMAC_SHA1);
        hmacEngine = new HMAC(MessageDigest.ALG_SHA);
        register();
    }

    @Override
    public void process(APDU apdu) throws ISOException {
        byte[] buf = apdu.getBuffer();

        if ((buf[ISO7816.OFFSET_CLA] == 0) && (buf[ISO7816.OFFSET_INS] == (byte) 0xA4)) {
            return;
        }

        if (buf[ISO7816.OFFSET_CLA] != CHALLENGE_CLA) {
            ISOException.throwIt(ISO7816.SW_CLA_NOT_SUPPORTED);
        }

        byte ins = buf[ISO7816.OFFSET_INS];

        switch (ins) {
            case KEYINSERT_INS:
                insert_key(apdu);
                break;
            case CHALLENGE_INS:
                key_challenge(apdu);
                break;
            default:
                ISOException.throwIt(ISO7816.SW_INS_NOT_SUPPORTED);
        }
    }

    private void insert_key(APDU apdu) {
        byte[] buf = apdu.getBuffer();

        serverKey.setKey(buf, apdu.getOffsetCdata(), (byte) apdu.getIncomingLength());
    }

    /*
    private void read_key(APDU apdu) {
        short le = apdu.setOutgoing();
        apdu.setOutgoingLength(KEY_LENGTH);
        if (le != KEY_LENGTH) {
            ISOException.throwIt(ISO7816.SW_WRONG_LENGTH);
        }
        byte param1 = apdu.getBuffer()[ISO7816.OFFSET_P1];
        if (param1 == AES_KEY) {
            serverAESKey.getKey(apdu.getBuffer(), (short) 0);
        } else if (param1 == HMAC_KEY) {
            serverHMACKey.getKey(apdu.getBuffer(), (short) 0);
        }
        apdu.sendBytes((short) 0, KEY_LENGTH);

    */
    private void key_challenge(APDU apdu) {
        short bytesReceived = apdu.getIncomingLength();
        byte[] buf = apdu.getBuffer();

        hmacEngine.init(serverKey, MessageDigest.ALG_SHA);
        short outputLength = hmacEngine.doFinal(buf, apdu.getOffsetCdata(), bytesReceived, apdu.getBuffer(), (short) 0);

        short le = apdu.setOutgoing();

        if (le != outputLength) {
            ISOException.throwIt(ISO7816.SW_WRONG_LENGTH);
        }
        apdu.setOutgoingLength(outputLength);
        apdu.sendBytes((short) 0, outputLength);
    }

    /*
    public static byte[] makeByteArray(short sNumber) {
        byte[] res = new byte[2];
        res[0] = (byte) ((sNumber >> 8) & 0xFF);
        res[1] = (byte) (sNumber & 0xFF);
        return res;
    }

    public static void copyShortToArray(short sNumber, byte[] array, short aOffset) {
        array[aOffset] = (byte) ((sNumber >> 8) & 0xFF);
        array[(short)(aOffset+1)] = (byte) (sNumber & 0xFF);
    }
    */
}